#!/usr/bin/perl -w

use strict;
use LWP::Simple;   # For getting RSS feed
use XML::RSS;      # For parsing RSS feed
use Text::Iconv;   # For converting from RSS encoding to email encoding
use Mail::Mailer;  # For sending mail
use MIME::Base64;  # For encoding Subject line

# Which SMTP server to use?
my $smtp_server = 'localhost';
# What to use in the From: field?
my $from = 'Me <me@here>';
# List of recepients
my @recepients = (
					'You <you@there>',
					'Him <him@somewhere>',
					'Her <her@anywhere>',
				);
# URL of the RSS feed
my $feed_link = 'http://www.livejournal.com/users/mamchenkov/data/rss';
# Temporary file to save RSS feed
my $feed_file = '/tmp/lj.feed.rss';
# Cache file to keep track of sent items
my $cache_file = '/tmp/lj.sent';
# Character encoding in RSS
my $from_enc = 'utf-8';
# Character encoding in email
my $to_enc   = 'koi-8';

# Init the converter
my $converter = Text::Iconv->new($from_enc,$to_enc);
# Init cache if it does not exist
open(CACHE,">$cache_file") unless (-e $cache_file);
close(CACHE);

# Read cache of sent items
my @cached_urls = ();
if (-e $cache_file) {
	open (CACHE,"<$cache_file");
	@cached_urls = <CACHE>;
	close(CACHE);
	chomp(@cached_urls);
}

# Get RSS feed
if (is_success(getstore($feed_link,$feed_file))) {
	my $rss = XML::RSS->new();
	$rss->parsefile($feed_file);
	# Inspect each item in the feed
	foreach my $item (@{ $rss->{'items'} }) {

		# Prepare item title to be used as Subject:
		my $title = $item->{'title'} || '';
		$title = $converter->convert($title);
		chomp($title);
		$title = '=?' . $to_enc . '?B?' . encode_base64($title,'') . '?=';

		# Clean up the body from HTML and stuff
		my $body  = $item->{'description'} || '';
		$body = $converter->convert($body);
		$body =~ s#<br />#\n#gi;
		chomp($body);

		# Prepare item link to be used in the body of the email
		my $url   = $item->{'link'} || '';
		$url = $converter->convert($url);
		chomp($url);

		# Do not proceed unless everything is in place
		next unless ($title && $body && $url);

		# If item is not in the cache
		unless (grep (/^$url$/, @cached_urls)) {
			# Send to item to each recepient
			foreach my $recepient (@recepients) {
				my $mailer = Mail::Mailer->new('smtp',Server=>$smtp_server);
				my %headers = (
								'Content-Type' => 'text/plain; charset=' . $to_enc,
								'To' => $recepient,
								'From' => $from,
								'Subject' => $title,
							);
				$mailer->open(\%headers);
				print $mailer "$body\n\nURL: $url\n";
				$mailer->close();
			}
			# Update cache
			open (CACHE, ">>$cache_file");
			print CACHE $url,"\n";
			close(CACHE);
		}
	}
}
else {
	die "Couldn't get RSS feed $feed_link\n";
}
